@@ -9,6 +9,7 @@ import numpy as np
 import pandas as pd
 import optuna
 
+from data_science_helper.model import general as g
 import data_science_helper.helper_classification_model as hcm
 import data_science_helper.model.neg_bagging_fraction__lgb_model as nbf_lgb_model
 
@@ -61,13 +62,31 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
     list_train_average_precision = []
     list_train_roc_auc = []
     
+    list_TP_train = []
+    list_FP_train = []
+    list_TN_train = []
+    list_FN_train = []
+    
+    list_TP_train_opt = []
+    list_FP_train_opt = []
+    list_TN_train_opt = []
+    list_FN_train_opt = []
+    
     list_valid_precision = []
     list_valid_recall = []
     list_valid_f1 = []
     list_valid_average_precision = []
     list_valid_roc_auc = []
     
+    list_TP_valid = []
+    list_FP_valid = []
+    list_TN_valid = []
+    list_FN_valid = []
     
+    list_TP_valid_opt = []
+    list_FP_valid_opt = []
+    list_TN_valid_opt = []
+    list_FN_valid_opt = []
     
     with_bias = False
     
@@ -93,8 +112,16 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
         list_train_average_precision.append(kpi["train/average_precision"])
         list_train_roc_auc.append(kpi["train/roc_auc"])        
         
+        list_TP_train.append(kpi["train/TP"])    
+        list_FP_train.append(kpi["train/FP"])  
+        list_TN_train.append(kpi["train/TN"])  
+        list_FN_train.append(kpi["train/FN"])  
         
-        
+        list_TP_train_opt.append(kpi["train/TP_opt"])    
+        list_FP_train_opt.append(kpi["train/FP_opt"])  
+        list_TN_train_opt.append(kpi["train/TN_opt"])  
+        list_FN_train_opt.append(kpi["train/FN_opt"]) 
+
         list_valid_score.append(kpi["valid/average_precision"])
         
         list_valid_precision.append(kpi["valid/precision"])
@@ -103,7 +130,15 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
         list_valid_average_precision.append(kpi["valid/average_precision"])
         list_valid_roc_auc.append(kpi["valid/roc_auc"])        
         
-
+        list_TP_valid.append(kpi["valid/TP"])    
+        list_FP_valid.append(kpi["valid/FP"])  
+        list_TN_valid.append(kpi["valid/TN"])  
+        list_FN_valid.append(kpi["valid/FN"])  
+        
+        list_TP_valid_opt.append(kpi["valid/TP_opt"])    
+        list_FP_valid_opt.append(kpi["valid/FP_opt"])  
+        list_TN_valid_opt.append(kpi["valid/TN_opt"])  
+        list_FN_valid_opt.append(kpi["valid/FN_opt"])  
         
         bias = kpi["bias"]
         
@@ -120,8 +155,54 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
     mean_valid_score = statistics.mean(list_valid_score)
     std_valid_score = statistics.pstdev(list_valid_score)
     
-
     
+    list_TP_train = []
+    list_FP_train = []
+    list_TN_train = []
+    list_FN_train = []
+    
+    
+    mean_TP_train = statistics.mean(list_TP_train)
+    std_TP_train = statistics.pstdev(list_TP_train)
+    mean_FP_train = statistics.mean(list_FP_train)
+    std_FP_train = statistics.pstdev(list_FP_train)
+    mean_TN_train = statistics.mean(list_TN_train)
+    std_TN_train = statistics.pstdev(list_TN_train)
+    mean_FN_train = statistics.mean(list_FN_train)
+    std_FN_train = statistics.pstdev(list_FN_train)
+    
+    mean_TP_valid = statistics.mean(list_TP_valid)
+    std_TP_valid = statistics.pstdev(list_TP_valid)
+    mean_FP_valid = statistics.mean(list_FP_valid)
+    std_FP_valid = statistics.pstdev(list_FP_valid)
+    mean_TN_valid = statistics.mean(list_TN_valid)
+    std_TN_valid = statistics.pstdev(list_TN_valid)
+    mean_FN_valid = statistics.mean(list_FN_valid)
+    std_FN_valid = statistics.pstdev(list_FN_valid)
+    
+    
+    
+    
+    mean_TP_train_opt = statistics.mean(list_TP_train_opt)
+    std_TP_train_opt = statistics.pstdev(list_TP_train_opt)
+    mean_FP_train_opt = statistics.mean(list_FP_train_opt)
+    std_FP_train_opt = statistics.pstdev(list_FP_train_opt)
+    mean_TN_train_opt = statistics.mean(list_TN_train_opt)
+    std_TN_train_opt = statistics.pstdev(list_TN_train_opt)
+    mean_FN_train_opt = statistics.mean(list_FN_train_opt)
+    std_FN_train_opt = statistics.pstdev(list_FN_train_opt)
+    
+    mean_TP_valid_opt = statistics.mean(list_TP_valid_opt)
+    std_TP_valid_opt = statistics.pstdev(list_TP_valid_opt)
+    mean_FP_valid_opt = statistics.mean(list_FP_valid_opt)
+    std_FP_valid_opt = statistics.pstdev(list_FP_valid_opt)
+    mean_TN_valid_opt = statistics.mean(list_TN_valid_opt)
+    std_TN_valid_opt = statistics.pstdev(list_TN_valid_opt)
+    mean_FN_valid_opt = statistics.mean(list_FN_valid_opt)
+    std_FN_valid_opt = statistics.pstdev(list_FN_valid_opt)
+    
+    
+
     mean_train_precision = statistics.mean(list_train_precision)           
     std_train_precision = statistics.pstdev(list_train_precision)
              
@@ -140,6 +221,7 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
     mean_valid_f1 = statistics.mean(list_valid_f1)           
     std_valid_f1 = statistics.pstdev(list_valid_f1)
     
+
     mean_train_average_precision = statistics.mean(list_train_average_precision) 
     std_train_average_precision = statistics.pstdev(list_train_average_precision)
     
@@ -185,7 +267,8 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
             split_valid_dic[key_split_valid] = list_valid_score[i] if with_bias==False else -1
             result_iter.update(split_valid_dic)
         
-        
+    
+ 
     result_kpis = {   
         
         'mean_train_precision':mean_train_precision,
@@ -199,6 +282,27 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
         'mean_train_roc_auc':mean_train_roc_auc,
         'std_train_roc_auc':std_train_roc_auc   ,
         
+        'mean_TP_train':mean_TP_train   ,
+        'std_TP_train':std_TP_train   ,
+        'mean_FP_train':mean_FP_train   ,
+        'std_FP_train':std_FP_train   ,
+        'mean_TN_train':mean_TN_train   ,
+        'std_TN_train':std_TN_train   ,
+        'mean_FN_train':mean_FN_train   ,
+        'std_FN_train':std_FN_train   ,
+        
+       
+         
+        'mean_TP_train_opt':mean_TP_train_opt   ,
+        'std_TP_train_opt':std_TP_train_opt   ,
+        'mean_FP_train_opt':mean_FP_train_opt   ,
+        'std_FP_train_opt':std_FP_train_opt   ,
+        'mean_TN_train_opt':mean_TN_train_opt   ,
+        'std_TN_train_opt':std_TN_train_opt   ,
+        'mean_FN_train_opt':mean_FN_train_opt   ,
+        'std_FN_train_opt':std_FN_train_opt   ,
+        
+        
         'mean_valid_precision':mean_valid_precision,
         'std_valid_precision':std_valid_precision,
         'mean_valid_recall':mean_valid_recall,
@@ -208,7 +312,28 @@ def objective_with_prune(trial: Trial, fast_check=True, X=None,y=None,fn_params=
         'mean_valid_average_precision':mean_valid_average_precision,
         'std_valid_average_precision':std_valid_average_precision,       
         'mean_valid_roc_auc':mean_valid_roc_auc,
-        'std_valid_roc_auc':std_valid_roc_auc       
+        'std_valid_roc_auc':std_valid_roc_auc   ,
+        
+        'mean_TP_valid':mean_TP_valid   ,
+        'std_TP_valid':std_TP_valid   ,
+        'mean_FP_valid':mean_FP_valid   ,
+        'std_FP_valid':std_FP_valid   ,
+        'mean_TN_valid':mean_TN_valid   ,
+        'std_TN_valid':std_TN_valid   ,
+        'mean_FN_valid':mean_FN_valid   ,
+        'std_FN_valid':std_FN_valid   ,
+        
+        
+        'mean_TP_valid_opt':mean_TP_valid_opt   ,
+        'std_TP_valid_opt':std_TP_valid_opt   ,
+        'mean_FP_valid_opt':mean_FP_valid_opt   ,
+        'std_FP_valid_opt':std_FP_valid_opt   ,
+        'mean_TN_valid_opt':mean_TN_valid_opt   ,
+        'std_TN_valid_opt':std_TN_valid_opt   ,
+        'mean_FN_valid_opt':mean_FN_valid_opt   ,
+        'std_FN_valid_opt':std_FN_valid_opt   ,        
+        
+        
     }
     
     result_iter.update(result_kpis)
@@ -253,13 +378,28 @@ def fit_lgbm_with_pruning(trial=None, train=None, val=None, fn_params=None, cat_
     # predictions
     
     y_pred_prob_train = model.predict(X_train, num_iteration=model.best_iteration)   
-    y_pred_prob_valid = model.predict(X_valid, num_iteration=model.best_iteration)   
+    y_pred_prob_valid = model.predict(X_valid, num_iteration=model.best_iteration) 
+    
+    
+    
+    y_pred_train = np.rint(y_pred_prob_train)    
+    opt_threshold_train  = g.get_opt_threshold(y_train,y_pred_prob_train)
+    
     
-    y_pred_train = np.rint(y_pred_prob_train)
     y_pred_valid = np.rint(y_pred_prob_valid)    
+    opt_threshold_valid  = g.get_opt_threshold(y_valid,y_pred_prob_valid)
+    print("11111111111111")
+    print(y_train.shape)
+    print("11111111111111")
 
+    TP_train, FP_train, TN_train, FN_train = perf_measure(y_train.values,y_pred_train)    
+    y_pred_train_opt = (y_pred_prob_train > opt_threshold_train).astype(int)
+    TP_train_opt, FP_train_opt, TN_train_opt, FN_train_opt = perf_measure(y_train.values,y_pred_train_opt)
     
-    #valid_average_precision2 = model.best_score['data_valid']['average_precision']
+    TP_valid, FP_valid, TN_valid, FN_valid = perf_measure(y_valid.values,y_pred_valid)
+    y_pred_valid_opt = (y_pred_prob_valid > opt_threshold_valid).astype(int)
+    TP_valid_opt, FP_valid_opt, TN_valid_opt, FN_valid_opt = perf_measure(y_valid.values,y_pred_valid_opt)
+
     
     train_average_precision = average_precision_score(y_train, y_pred_prob_train)
     valid_average_precision = average_precision_score(y_valid, y_pred_prob_valid)
@@ -279,20 +419,60 @@ def fit_lgbm_with_pruning(trial=None, train=None, val=None, fn_params=None, cat_
            'train/recall': recall_t,
            'train/f1': f1_t,
            'train/roc_auc': roc_auc_t,        
-        
-          
+           'train/TP': TP_train,   
+           'train/FP': FP_train,   
+           'train/TN': TN_train,   
+           'train/FN': FN_train,   
+           
+           'train/opt_threshold_train': opt_threshold_train,  
+           
+           
+           'train/TP_opt': TP_train_opt,   
+           'train/FP_opt': FP_train_opt,   
+           'train/TN_opt': TN_train_opt,   
+           'train/FN_opt': FN_train_opt,  
+           
            'valid/average_precision': valid_average_precision,
            'valid/precision': precision,
            'valid/recall': recall,
            'valid/f1': f1,
            'valid/roc_auc': roc_auc,
+           'valid/TP': TP_valid,   
+           'valid/FP': FP_valid,   
+           'valid/TN': TN_valid,   
+           'valid/FN': FN_valid,   
+           
+           'valid/TP_opt': TP_valid_opt,   
+           'valid/FP_opt': FP_valid_opt,   
+           'valid/TN_opt': TN_valid_opt,   
+           'valid/FN_opt': FN_valid_opt,  
+           
+           'valid/opt_threshold_valid': opt_threshold_valid, 
            
            'bias':bias
           }
     return model, kpi
 
 
+def perf_measure(y_actual, y_hat):
+    
+    TP = 0
+    FP = 0
+    TN = 0
+    FN = 0
+    
+    for i in range(len(y_hat)): 
+        
+        if y_actual[i]==y_hat[i]==1:
+           TP += 1
+        if y_hat[i]==1 and y_actual[i]!=y_hat[i]:
+           FP += 1
+        if y_actual[i]==y_hat[i]==0:
+           TN += 1
+        if y_hat[i]==0 and y_actual[i]!=y_hat[i]:
+           FN += 1
 
+    return(TP, FP, TN, FN)
   
 
 def lgb_model_optuna(X_train=None, y_train=None,n_trials=2,fast_check=True,fn_params=None, 
